@@ -15,6 +15,8 @@ use crate::executor::{AppResponse, Executor};
 use crate::transactions::transactional;
 use crate::wasm::{ContractData, Wasm, WasmKeeper};
 
+use anyhow::Result as AnyResult;
+
 pub fn next_block(block: &mut BlockInfo) {
     block.time = block.time.plus_seconds(5);
     block.height += 1;
@@ -51,7 +53,7 @@ impl<C> Executor<C> for App<C>
 where
     C: Clone + fmt::Debug + PartialEq + JsonSchema + 'static,
 {
-    fn execute(&mut self, sender: Addr, msg: CosmosMsg<C>) -> Result<AppResponse, String> {
+    fn execute(&mut self, sender: Addr, msg: CosmosMsg<C>) -> AnyResult<AppResponse> {
         let mut all = self.execute_multi(sender, vec![msg])?;
         let res = all.pop().unwrap();
         Ok(res)
@@ -103,7 +105,7 @@ where
         &mut self,
         sender: Addr,
         msgs: Vec<CosmosMsg<C>>,
-    ) -> Result<Vec<AppResponse>, String> {
+    ) -> AnyResult<Vec<AppResponse>> {
         // we need to do some caching of storage here, once in the entry point:
         // meaning, wrap current state, all writes go to a cache, only when execute
         // returns a success do we flush it (otherwise drop it)
@@ -123,7 +125,7 @@ where
     }
 
     /// This is an "admin" function to let us adjust bank accounts
-    pub fn init_bank_balance(&mut self, account: &Addr, amount: Vec<Coin>) -> Result<(), String> {
+    pub fn init_bank_balance(&mut self, account: &Addr, amount: Vec<Coin>) -> AnyResult<()> {
         self.router
             .bank
             .init_balance(&mut *self.storage, account, amount)
@@ -136,7 +138,7 @@ where
     }
 
     /// This allows to get `ContractData` for specific contract
-    pub fn contract_data(&self, address: &Addr) -> Result<ContractData, String> {
+    pub fn contract_data(&self, address: &Addr) -> AnyResult<ContractData> {
         self.router.wasm.contract_data(&*self.storage, address)
     }
 
@@ -147,8 +149,8 @@ where
         &mut self,
         contract_addr: U,
         msg: &T,
-    ) -> Result<AppResponse, String> {
-        let msg = to_vec(msg).map_err(|e| e.to_string())?;
+    ) -> AnyResult<AppResponse> {
+        let msg = to_vec(msg)?;
         self.router.wasm.sudo(
             &*self.api,
             contract_addr.into(),
@@ -199,7 +201,7 @@ where
         storage: &dyn Storage,
         block: &BlockInfo,
         request: QueryRequest<Empty>,
-    ) -> Result<Binary, String> {
+    ) -> AnyResult<Binary> {
         match request {
             QueryRequest::Wasm(req) => {
                 self.wasm
@@ -217,7 +219,7 @@ where
         block: &BlockInfo,
         sender: Addr,
         msg: CosmosMsg<C>,
-    ) -> Result<AppResponse, String> {
+    ) -> AnyResult<AppResponse> {
         match msg {
             CosmosMsg::Wasm(msg) => self.wasm.execute(api, storage, &self, block, sender, msg),
             CosmosMsg::Bank(msg) => self.bank.execute(storage, sender, msg),
@@ -285,10 +287,12 @@ mod test {
         StdResult, SubMsg, WasmMsg,
     };
 
+    use crate::error::Error;
     use crate::test_helpers::contracts::{echo, hackatom, payout, reflect};
     use crate::test_helpers::{CustomMsg, EmptyMsg};
     use crate::transactions::StorageTransaction;
     use crate::BankKeeper;
+    use cosmwasm_std::{OverflowError, OverflowOperation, StdError};
 
     use super::*;
 
@@ -617,7 +621,10 @@ mod test {
         let err = app
             .execute_contract(random.clone(), reflect_addr.clone(), &msgs, &[])
             .unwrap_err();
-        assert_eq!("Overflow: Cannot Sub with 0 and 3", err.as_str());
+        assert_eq!(
+            StdError::overflow(OverflowError::new(OverflowOperation::Sub, 0, 3)),
+            err.downcast().unwrap()
+        );
 
         // first one should have been rolled-back on error (no second payment)
         let funds = get_balance(&app, &random);
@@ -710,45 +717,23 @@ mod test {
 
         // expected events: execute, transfer, reply, custom wasm (set in reply)
         assert_eq!(4, res.events.len(), "{:?}", res.events);
-        let first = &res.events[0];
-        assert_eq!(first.ty.as_str(), "execute");
-        assert_eq!(first.attributes, [("_contract_addr", &reflect_addr)]);
-
-        // next event is the transfer from bank
-        let transfer = &res.events[1];
-        assert_eq!(transfer.ty.as_str(), "transfer");
-
-        // then we get notification reply was called
-        let reply = &res.events[2];
-        assert_eq!(reply.ty.as_str(), "reply");
-        assert_eq!(
-            reply.attributes,
-            [
-                ("_contract_addr", reflect_addr.as_str()),
-                ("mode", "handle_success")
-            ]
-        );
-
-        // the last one is a custom event (from reply)
-        let custom = &res.events[3];
-        assert_eq!("wasm-custom", custom.ty.as_str());
-        assert_eq!(
-            custom.attributes,
-            [
-                // TODO
-                ("_contract_addr", reflect_addr.as_str()),
-                ("from", "reply"),
-                ("to", "test")
-            ]
+        res.assert_event(&Event::new("execute").add_attribute("_contract_addr", &reflect_addr));
+        res.assert_event(&Event::new("transfer").add_attribute("amount", "7eth"));
+        res.assert_event(
+            &Event::new("reply")
+                .add_attribute("_contract_addr", reflect_addr.as_str())
+                .add_attribute("mode", "handle_success"),
         );
+        res.assert_event(&Event::new("wasm-custom").add_attribute("from", "reply"));
 
         // ensure success was written
         let res: Reply = app.wrap().query_wasm_smart(&reflect_addr, &query).unwrap();
         assert_eq!(res.id, 123);
         // validate the events written in the reply blob...should just be bank transfer
-        let reply_events = res.result.unwrap().events;
-        assert_eq!(1, reply_events.len());
-        assert_eq!("transfer", &reply_events[0].ty);
+        let reply = res.result.unwrap();
+        assert_eq!(1, reply.events.len());
+        AppResponse::from(reply)
+            .assert_event(&Event::new("transfer").add_attribute("amount", "7eth"));
 
         // reflect sends 300 btc, failure, but error caught by submessage (so shows success)
         let msg = SubMsg::reply_always(
@@ -1185,16 +1170,16 @@ mod test {
             };
 
             let res = app
-                .execute_contract(owner, reflect_addr, &reflect_msg, &[])
+                .execute_contract(owner, reflect_addr.clone(), &reflect_msg, &[])
                 .unwrap();
 
             // ensure data is empty
             assert_eq!(res.data, None);
             // ensure expected events
             assert_eq!(res.events.len(), 3, "{:?}", res.events);
-            assert_eq!("execute", &res.events[0].ty);
-            assert_eq!("execute", &res.events[1].ty);
-            assert_eq!("wasm-echo", &res.events[2].ty);
+            res.assert_event(&Event::new("execute").add_attribute("_contract_addr", &reflect_addr));
+            res.assert_event(&Event::new("execute").add_attribute("_contract_addr", &echo_addr));
+            res.assert_event(&Event::new("wasm-echo"));
         }
 
         #[test]
@@ -1366,7 +1351,7 @@ mod test {
                 )
                 .unwrap_err();
 
-            assert_eq!(err, "Empty attribute key. Value: value");
+            assert_eq!(Error::empty_attribute_key("value"), err.downcast().unwrap(),);
         }
 
         #[test]
@@ -1396,7 +1381,7 @@ mod test {
                 )
                 .unwrap_err();
 
-            assert_eq!(err, "Empty attribute value. Key: key");
+            assert_eq!(Error::empty_attribute_value("key"), err.downcast().unwrap());
         }
 
         #[test]
@@ -1425,7 +1410,7 @@ mod test {
                 )
                 .unwrap_err();
 
-            assert_eq!(err, "Empty attribute key. Value: value");
+            assert_eq!(Error::empty_attribute_key("value"), err.downcast().unwrap());
         }
 
         #[test]
@@ -1454,7 +1439,7 @@ mod test {
                 )
                 .unwrap_err();
 
-            assert_eq!(err, "Empty attribute value. Key: key");
+            assert_eq!(Error::empty_attribute_value("key"), err.downcast().unwrap());
         }
 
         #[test]
@@ -1481,7 +1466,7 @@ mod test {
                 )
                 .unwrap_err();
 
-            assert_eq!(err, "Event type too short: e");
+            assert_eq!(Error::event_type_too_short("e"), err.downcast().unwrap());
         }
     }
 }
